<?php

class actionIbackuperRestore extends cmsAction {

    public function run() {

        $form = $this->getForm('restore');

        if ($this->request->has('submit')){

            // загружаем файл
            $form_items = $form->parse($this->request, true);
            $errors = $form->validate($this, $form_items);

            if (empty($form_items['file_restore']['path']) && empty($errors['file_restore'])){
                $errors['file_restore'] = LANG_IBACKUPER_FILE_NOT_LOAD;
            }

            if (!$errors){

                // получаем содержимое файла
                $data = cmsModel::yamlToArray(file_get_contents(cmsConfig::get('upload_path') . $form_items['file_restore']['path']));

                // удаляем файл
                if (!empty($form_items['file_restore']['id'])){
                    cmsCore::getModel('files')->deleteFile($form_items['file_restore']['id']);
                }

                if (!isset($data['ctype']) && !isset($data['form'])){

                    $errors['file_restore'] = LANG_IBACKUPER_EMPTY_FILE;

                }

                if (isset($data['ctype']) && !$errors){

                    $data['ctype']['name'] = !empty($form_items['new_name']) ? $form_items['new_name'] : $data['ctype']['name'];

                    $data['ctype']['name'] = str_replace('-', '_', $data['ctype']['name']);

                    if(cmsCore::isControllerExists($data['ctype']['name'])){
                        $errors['new_name'] = LANG_CP_CTYPE_ERROR_NAME;
                    }

                    cmsCore::loadControllerLanguage('content');
                    $content_model = cmsCore::getModel('content');

                    // определяем, есть ли такой тип контента
                    $use_prefix = $content_model->getContentTypeByName($data['ctype']['name']) ? true : false;

                    // если такой тип контента уже есть, используем префикс в названии.
                    if ($use_prefix){

                        $data['ctype']['name'] = 'new_' .  $data['ctype']['name'];

                        $isset = $content_model->getContentTypeByName($data['ctype']['name']);

                        if ($isset){
                            $errors['new_name'] = LANG_IBACKUPER_CTYPE_ISSET;
                        }

                    }

                    if (!$errors){

                        // удаляем id старого типа контента и снимаем блокировку с нового типа контента
                        unset($data['ctype']['id']);
                        $data['ctype']['is_fixed'] = NULL;

                        if (empty($form_items['is_relations'])){
                            unset($data['ctype']['relations']);
                        }

                        $data['ctype']['title'] = !empty($form_items['new_title']) ? $form_items['new_title'] : $data['ctype']['title'];

                        $data['ctype']['labels']['one'] = !empty($form_items['new_labels_one']) ? $form_items['new_labels_one'] : $data['ctype']['labels']['one'];

                        $data['ctype']['labels']['two'] = !empty($form_items['new_labels_two']) ? $form_items['new_labels_two'] : $data['ctype']['labels']['two'];

                        $data['ctype']['labels']['many'] = !empty($form_items['new_labels_many']) ? $form_items['new_labels_many'] : $data['ctype']['labels']['many'];

                        $data['ctype']['labels']['create'] = !empty($form_items['new_labels_create']) ? $form_items['new_labels_create'] : $data['ctype']['labels']['create'];

                        $data['ctype']['labels']['list'] = !empty($form_items['new_labels_list']) ? $form_items['new_labels_list'] : $data['ctype']['labels']['list'];

                        $data['ctype']['labels']['profile'] = !empty($form_items['new_labels_profile']) ? $form_items['new_labels_profile'] : $data['ctype']['labels']['profile'];

                        $data['ctype'] = cmsEventsManager::hook('ctype_before_add', $data['ctype']);
                        $data['ctype'] = cmsEventsManager::hook("ctype_{$data['ctype']['name']}_before_add", $data['ctype']);

                        // добавляем новый тип контента
                        $ctype_id = $content_model->addContentType($data['ctype']);
                        $data['ctype']['id']=$ctype_id;

                        cmsEventsManager::hook('ctype_after_add', $data['ctype']);
                        cmsEventsManager::hook("ctype_{$data['ctype']['name']}_after_add", $data['ctype']);

                        if ($ctype_id){
                            cmsCore::getController('admin')->addCtypeWidgetsPages($data['ctype']);
                        }

                        // добавляем категории
                        if (!empty($data['cats'])){

                            ksort($data['cats']);

                            $max_ns_right = 1;

                            foreach ($data['cats'] as $key => $value){

                                $max_ns_right = $value['ns_right'] > $max_ns_right ? $value['ns_right'] : $max_ns_right;

                                if ($value['ns_differ'] == NULL){unset($value['ns_differ']);}

                                $content_model->insert('con_' . $data['ctype']['name'] . '_cats', $value);
                            }

                            $content_model->update('con_' . $data['ctype']['name'] . '_cats', 1, array('ns_right' => $max_ns_right + 1));

                        }

                        // добавляем поля
                        if (!empty($data['fields'])){

                            $fields_types = cmsForm::getAvailableFormFields(false);

                            foreach ($data['fields'] as $name => $field){

                                unset($data['fields'][$name]['handler'], $data['fields'][$name]['handler_title']);

                                if (!array_key_exists($field['type'], $fields_types) || !isset($field['type'])){
                                    if ($content_model->db->isFieldExists($content_model->table_prefix . $data['ctype']['name'], $name)){
                                        $content_model->db->dropTableField($content_model->table_prefix . $data['ctype']['name'], $name);
                                    }
                                    unset($data['fields'][$name]); continue;
                                }

                                if ($field['type'] == 'parent'){
                                    unset($data['fields'][$name]); continue;
                                }

                                $data['fields'][$name]['ctype_id'] = $ctype_id;

                            }

                            $this->model->addCtypeFields($data['ctype']['name'], $data['fields']);

                        }

                        // добавляем свойства
                        if (!empty($data['props'])){

                            foreach ($data['props'] as $id => $prop){

                                $prop['ctype_id'] = $ctype_id;

                                if ($prop['options'] == '---'){$prop['options'] = null;}

                                $content_model->insert($content_model->table_prefix . $data['ctype']['name'] . '_props', $prop);

                                cmsEventsManager::hook('ctype_prop_after_add', array($prop, $data['ctype']['name'], $content_model));

                            }

                        }

                        // добавляем связи свойств с категориями
                        if (!empty($data['props_bind'])){

                            foreach ($data['props_bind'] as $id => $prop){

                                $content_model->insert($content_model->table_prefix . $data['ctype']['name'] . '_props_bind', array(
                                    'prop_id' => $prop['prop_id'],
                                    'cat_id' => $prop['cat_id'],
                                    'ordering' => $prop['ordering']
                                ));

                            }

                        }

                        // добавляем наборы
                        if (!empty($data['datasets'])){

                            $fields_names = array_keys($data['fields']);

                            foreach ($data['datasets'] as $name => $dataset) {

                                if ($dataset['filters']){

                                    foreach ($dataset['filters'] as $id => $filters){
                                        if (!in_array($filters['field'], $fields_names)){
                                            unset($dataset['filters'][$id]);
                                        }
                                    }

                                }


                                if ($dataset['sorting']){

                                    foreach ($dataset['sorting'] as $id => $sorting){
                                        if (!in_array($sorting['by'], $fields_names)){
                                            unset($dataset['sorting'][$id]);
                                        }
                                    }

                                }

                                unset($dataset['id']);
                                $dataset['id'] = $content_model->addContentDataset($dataset, $data['ctype']);
                            }

                        }

                        // добавляем модераторов
                        if (!empty($data['moderators'])){

                            $users_model = cmsCore::getModel('users');

                            foreach ($data['moderators'] as $id => $moderator){

                                $users_model->filterEqual('nickname', $moderator['user_nickname']);
                                $users_model->filterEqual('id', $moderator['user_id']);
                                $users_ids = $users_model->getUsersIds();

                                if(!empty($users_ids)){

                                    $this->model->insert('moderators', array(
                                        'ctype_name' => $data['ctype']['name'],
                                        'user_id' => $moderator['user_id'],
                                        'date_assigned' => ''
                                    ));

                                }

                            }

                        }

                        // добавляем связи
                        if (!empty($form_items['is_relations']) && !empty($data['relations'])){

                            foreach ($data['relations'] as $key => $relation) {

                                $relation['ctype_id'] = $ctype_id;
                                unset($relation['id'], $relation['ordering']);

                                if($relation['target_controller'] != 'content'){

                                    $content_model->setTablePrefix('');

                                    cmsCore::loadControllerLanguage($relation['target_controller']);
                                    $target_ctype = array(
                                        'title' => string_lang('LANG_'.strtoupper($relation['target_controller']).'_CONTROLLER'),
                                        'name' => $relation['target_controller'],
                                        'id'  => null
                                    );

                                } else {

                                    $target_ctype = $content_model->getContentType($relation['child_ctype_id']);
                                }

                                if (empty($target_ctype['name'])){ continue; }

                                if ($target_ctype['title'] != $relation['title']){ continue; }


                                $parent_field_name = "parent_{$data['ctype']['name']}_id";

                                if (!$content_model->isContentFieldExists($target_ctype['name'], $parent_field_name)){
                                    $content_model->addContentField($target_ctype['name'], array(
                                        'type'          => 'parent',
                                        'ctype_id'      => $target_ctype['id'],
                                        'name'          => $parent_field_name,
                                        'title'         => mb_convert_case($data['ctype']['labels']['one'], MB_CASE_TITLE),
                                        'options'       => array(),
                                        'is_fixed'      => true,
                                        'is_in_filter'  => false,
                                        'is_fixed_type' => true
                                    ));

                                }

                                if ($content_model->table_prefix == ''){
                                    $content_model->setTablePrefix('con_');
                                }

                                $relation_id = $content_model->addContentRelation($relation);

                            }

                        }

                        // добавляем доступы
                        if (!empty($data['perm_values'])){
                            cmsPermissions::savePermissions($data['ctype']['name'], $data['perm_values']);
                        }

                        // добавляем настройки создания карты сайта
                        if (!empty($data['sitemap_options'])){
                            $sitemap_options = cmsController::loadOptions('sitemap');
                            $sitemap_options['sources']['[content|' . $data['ctype']['name'] . ']'] = 1;
                            cmsController::saveOptions('sitemap', $sitemap_options);
                        }

                        // добавляем настройку в типы активности
                        $activity_controller = cmsCore::getController('activity');

                        if ($activity_controller->isTypeExists('content', "add.{$data['ctype']['name']}")){

                            $activity_controller->updateType('content', "add.{$data['ctype']['name']}", array(
                                'title' => sprintf(LANG_IBACKUPER_CONTENT_ACTIVITY_ADD, $data['ctype']['labels']['many']),
                                'description' => sprintf(LANG_IBACKUPER_CONTENT_ACTIVITY_ADD_DESC, $data['ctype']['labels']['create'], '%s')
                            ));

                        } else {

                            $activity_controller->addType(array(
                                'controller'  => 'content',
                                'name'        => "add.{$data['ctype']['name']}",
                                'is_enabled'  => 0,
                                'title'       => sprintf(LANG_CONTENT_ACTIVITY_ADD, $data['ctype']['labels']['many']),
                                'description' => sprintf(LANG_CONTENT_ACTIVITY_ADD_DESC, $data['ctype']['labels']['create'], '%s')
                            ));

                        }

                        // добавляем пресеты изображений, если их нет
                        if (!empty($data['images_presets'])){

                            $presets = $this->model->get('images_presets', function($item, $model){
                                return $item['title'];
                            }, 'name');

                            if ($presets){

                                foreach ($presets as $name => $title) {
                                    if (!empty($data['images_presets'][$name])){ unset($data['images_presets'][$name]); }
                                }

                                if (!empty($data['images_presets'])){

                                    foreach ($data['images_presets'] as $name => $value){
                                        $this->model->insert('images_presets', $value);
                                    }

                                }

                            }

                        }

                        cmsUser::addSessionMessage(sprintf(LANG_IBACKUPER_CTYPE_ADD_SUCCESS, $data['ctype']['name'], $data['ctype']['title']), 'success');

                    }

                }

                if (isset($data['form']) && !$errors){

                    $data['form']['name'] = !empty($form_items['new_name']) ? $form_items['new_name'] : $data['form']['name'];

                    // определяем, есть ли такая форма
                    $use_prefix = $this->model->filterEqual('name', $data['form']['name'])->getCount('iforms');
                    $this->model->resetFilters();

                    if ($use_prefix){
                        $data['form']['name'] = 'new_' .  $data['form']['name'];

                        $isset = $this->model->filterEqual('name', $data['form']['name'])->getCount('iforms');
                        if ($isset){
                            $errors['new_name'] = LANG_IBACKUPER_IFORMS_ISSET;
                        }

                    }

                    if (!$errors){

                        $data['form']['title'] = !empty($form_items['new_title']) ? $form_items['new_title'] : $data['form']['title'];

                        $iforms_model = cmsCore::getModel('iforms');

                        unset($data['form']['id']);

                        if (isset($data['form']['content'])){

                            $data['form']['tpl_messages'] = array();

                            if (!is_array($data['form']['users'])){ $data['form']['users'] = cmsModel::yamlToArray($data['form']['users']); }
                            if (!is_array($data['form']['options'])){ $data['form']['options'] = cmsModel::yamlToArray($data['form']['options']); }

                            if ($data['form']['users']['pm_email'] == 'pm'){
                                    $data['form']['options']['pm'] = 1;
                            } elseif ($data['form']['users']['pm_email'] == 'email') {
                                    $data['form']['options']['email']['is_enable'] = 1;
                            } else {
                                    $data['form']['options']['pm'] = 1;
                                    $data['form']['options']['email']['is_enable'] = 1;
                            }

                            $data['form']['options']['email']['from'] = $data['form']['options']['email_from'];
$data['form']['options']['email']['count_to_queue'] = 0;
                            $data['form']['tpl_messages']['for_email'] = $data['form']['options']['letter_tpl'];
                            $data['form']['tpl_messages']['for_notice_email'] = $data['form']['options']['send_notice_tpl'];

                            if (isset($data['form']['content']['author'])){$data['form']['users']['content_author'] = $data['form']['content']['author'];}
                            if (isset($data['form']['content']['owner'])){$data['form']['users']['content_owner'] = $data['form']['content']['owner'];}
                            if (isset($data['form']['content']['members'])){$data['form']['users']['content_members'] = $data['form']['content']['members'];}
                            if (isset($data['form']['group_ids'])){$data['form']['users']['group_ids'] = $data['form']['group_ids'];}

                        }

                        $form_id = $iforms_model->addIformsForm($data['form']);

                        if (!empty($data['fields'])){

                            foreach ($data['fields'] as $name => $field){

                                unset($field['id']);

                                $field['iforms_id'] = $form_id;

                                $field_id = $iforms_model->addFormField('iforms_fields', $field);

                            }

                        }

                        cmsUser::addSessionMessage(sprintf(LANG_IBACKUPER_IFORMS_ADD_SUCCESS, $data['form']['name'], $data['form']['title']), 'success');

                    }

                }

                $this->redirectBack();

            }

            if ($errors){
                cmsUser::addSessionMessage(LANG_FORM_ERRORS, 'error');
            }

        }

        // выводим форму
        return $this->cms_template->render('backend/restore', array(
            'form' => $form,
            'form_items' => isset($form_items) ? $form_items : array(),
            'errors' => isset($errors) ? $errors : false,
        ));

    }

}
